local require = GLOBAL.require
local SpawnPrefab = GLOBAL.SpawnPrefab
--
require "class"
--
local Ingredient = GLOBAL.Ingredient
local RECIPETABS = GLOBAL.RECIPETABS
local STRINGS = GLOBAL.STRINGS
local ACTIONS = GLOBAL.ACTIONS
local Action = GLOBAL.Action

PrefabFiles = {
	"batter",
	"ofbat",
	"alpha",
	"omega",
	"epsilon",
	"addons",
}
GLOBAL.STRINGS.NAMES.OFBAT = "Harold Bat"
GLOBAL.STRINGS.CHARACTERS.GENERIC.DESCRIBE.OFBAT = "This is my bat. It's my offensive equipment."
GLOBAL.STRINGS.NAMES.ALPHA = "Alpha"
GLOBAL.STRINGS.CHARACTERS.GENERIC.DESCRIBE.ALPHA = "Alpha, the Father. A bizarre object, associated with plastic."
GLOBAL.STRINGS.NAMES.OMEGA = "Omega"
GLOBAL.STRINGS.CHARACTERS.GENERIC.DESCRIBE.OMEGA = "Omega, the Son. A bizarre object, associated with metal."
GLOBAL.STRINGS.NAMES.EPSILON = "Epsilon"
GLOBAL.STRINGS.CHARACTERS.GENERIC.DESCRIBE.EPSILON = "Epsilon, the Holy Spirit. A bizarre object, associated with smoke."
GLOBAL.STRINGS.NAMES.ADDONS = "Add-ons"
GLOBAL.STRINGS.CHARACTERS.GENERIC.DESCRIBE.ADDONS = "All of my add-ons. A bizarre collection of objects, associated with the elements."
Assets = {
    Asset( "IMAGE", "images/saveslot_portraits/batter.tex" ),
    Asset( "ATLAS", "images/saveslot_portraits/batter.xml" ),
    Asset( "IMAGE", "images/selectscreen_portraits/batter.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/batter.xml" ),
    Asset( "IMAGE", "images/selectscreen_portraits/batter_silho.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/batter_silho.xml" ),
    Asset( "IMAGE", "bigportraits/batter.tex" ),
    Asset( "ATLAS", "bigportraits/batter.xml" ),
	
	Asset("SOUNDPACKAGE", "sound/batter.fev"),
	Asset("SOUND", "sound/batter.fsb"),
	
	Asset("IMAGE", "minimap/batter.tex"),
	Asset("ATLAS", "minimap/batter.xml"),
    AddMinimapAtlas("minimap/batter.xml"),
}
RemapSoundEvent( "dontstarve/characters/batter/death_voice", "batter/characters/batter/death_voice" )
RemapSoundEvent( "dontstarve/characters/batter/hurt", "batter/characters/batter/hurt" )
RemapSoundEvent( "dontstarve/characters/batter/talk_LP", "batter/characters/batter/talk_LP" )
---Music replacements
RemapSoundEvent( "dontstarve/music/music_work", "batter/music/work" )
RemapSoundEvent( "dontstarve/music/music_work_cave", "batter/music/work_cave" )
RemapSoundEvent( "dontstarve/music/music_danger", "batter/music/danger" )
RemapSoundEvent( "dontstarve/music/music_danger_cave", "batter/music/danger_cave" )
RemapSoundEvent( "dontstarve/music/music_epicfight", "batter/music/epicfight" )
local DynamicMusic = Class(function(self, inst)
    self.inst = inst
    self.inst:StartUpdatingComponent(self)
    
    self.enabled = true

    self.is_busy = false
    self.busy_timeout = 0
    
    self.playing_danger = false
    
    if GetWorld():IsCave() then
		self.inst.SoundEmitter:PlaySound( "dontstarve/music/music_work_cave", "busy")
	else
		self.inst.SoundEmitter:PlaySound( "dontstarve/music/music_work", "busy")
	end
	
    self.inst.SoundEmitter:SetParameter( "busy", "intensity", 0 )
    
    self.inst:ListenForEvent( "gotnewitem", function() self:OnContinueBusy() end )  
    self.inst:ListenForEvent( "dropitem", function() self:OnContinueBusy() end )  
    
    self.inst:ListenForEvent( "attacked", function(inst, dat)
        if self.enabled
           and dat.attacker
           and dat.attacker ~= self.inst
           and not dat.attacker:HasTag("shadow") then
            self:OnStartDanger()
        end
    end )  
    self.inst:ListenForEvent( "doattack", function(inst, dat)
		if self.enabled
            and dat
            and dat.target
		    and not dat.target:HasTag("prey")
			and not dat.target:HasTag("bird")
			and not dat.target:HasTag("wall")
			and not dat.target:HasTag("butterfly")
			and not dat.target:HasTag("shadow")
			and not dat.target:HasTag("veggie")
            and not dat.target:HasTag("smashable") then
			self:OnStartDanger()
		end
	end )  
    self.inst:ListenForEvent( "resurrect", function(inst)
        self:StopPlayingDanger()
    end)
    
  
    self.inst:ListenForEvent( "dusktime", function(it, data) 
            
            if self.enabled and 
                not self.playing_danger then
                self:StopPlayingBusy()
                self.inst.SoundEmitter:PlaySound( "dontstarve/music/music_dusk_stinger")
            end
            
            
        end, GetWorld())      

    self.inst:ListenForEvent( "daytime", function(it, data) 
    
            if self.enabled and 
                data.day > 0 and not self.playing_danger then
                self:StopPlayingBusy()
                self.inst.SoundEmitter:PlaySound( "dontstarve/music/music_dawn_stinger")
            end
            
        end, GetWorld())      

    self.inst:ListenForEvent( "nighttime", function(it, data) 
        end, GetWorld())      
    
    
    inst:ListenForEvent( "builditem", function(it, data) self:OnStartBusy() end)  
    inst:ListenForEvent( "buildstructure", function(it, data) self:OnStartBusy() end)  
    inst:ListenForEvent( "working", function(it, data) self:OnStartBusy() end)  
    
    
end)

function DynamicMusic:OnStartDanger()

    if not self.enabled then return end
    
    self.danger_timeout = 10
    if not self.playing_danger then
        local epic = GetClosestInstWithTag("epic", self.inst, 30)
        local soundpath = nil
        
        if epic then
            soundpath = "batter/music/epicfight"
        elseif GetWorld():IsCave() then
            soundpath = "batter/music/danger_cave"
        else
            soundpath = "batter/music/danger"
        end

        self.inst.SoundEmitter:PlaySound(soundpath, "danger")
        self:StopPlayingBusy()
        self.playing_danger = true
    end
end


function BatterPostInit(batter)
--Recipes
        local alpharecipe = GLOBAL.Recipe( ("alpha"), { Ingredient("twigs", 3),Ingredient("spidergland", 3),Ingredient("rocks", 4)}, RECIPETABS.REFINE,  {SCIENCE = 0} )
        alpharecipe.atlas = "images/inventoryimages/alpha.xml"
        local omegarecipe = GLOBAL.Recipe( ("omega"), { Ingredient("log", 3),Ingredient("goldnugget", 4),Ingredient("shovel", 1)}, RECIPETABS.REFINE,  {SCIENCE = 1} )
        omegarecipe.atlas = "images/inventoryimages/omega.xml"
		local epsilonrecipe = GLOBAL.Recipe( ("epsilon"), { Ingredient("boards", 3),Ingredient("nitre", 4),Ingredient("charcoal", 1)}, RECIPETABS.REFINE,  {SCIENCE = 2} )
        epsilonrecipe.atlas = "images/inventoryimages/epsilon.xml"
		local addonsrecipe = GLOBAL.Recipe( ("addons"), { Ingredient("alpha", 1),Ingredient("omega", 1),Ingredient("epsilon", 1),Ingredient("cookedmeat", 1),Ingredient("honey", 5)}, RECIPETABS.REFINE,  {SCIENCE = 2} )
        addonsrecipe.atlas = "images/inventoryimages/addons.xml"
end

AddSimPostInit(function(inst)
        if inst.prefab == "batter" then
                BatterPostInit(inst)
        end
end)

GetPlayer = GLOBAL.GetPlayer
table.insert(GLOBAL.CHARACTER_GENDERS.MALE, "batter")
AddModCharacter("batter")